 /* mbed UniGraphic library - Device specific class
 * Copyright (c) 2015 Giuliano Dianda
 * Released under the MIT License: http://mbed.org/license/mit
 */
#include "Protocols.h"
#include "ST7565.h"

/*adapted to S1D15721*/

/////////////////////////////////////////////////////////////////////////
// display settings /////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
#define IC_X_SEGS    240 // ST7565 SEG has range 0-131 (131-0 if ADC=1), check your datasheet, important for the orientation
#define IC_Y_COMS    64  // ST7565 COM has range 0-63 (63-0 if SHL=1), check your datasheet, important for the orientation

#define LCD_CMD_DATA_WRITE          0x1D    // Display Data Write (n-bytes)
#define LCD_CMD_PAGE_ADDR           0xB1    // Page Address Set (2 byte)
#define LCD_CMD_COL_ADDR            0x13    // Column Address Set (2 byte)
#define LCD_CMD_DISPLAY_OFFON       0xAE    // Display ON/OFF: 0=Off/1=On                             // =ST7565
#define LCD_CMD_DISPLAY_REVERSE     0xA6    // Display/Normal: 0=Normal/1=Reverse                     // =ST7565
#define LCD_CMD_DISPLAY_ALL_LIGHT   0xA4    // Display All Lighting: 0=Normal/1=All On                // =ST7565
#define LCD_CMD_DISPLAY_START_LINE  0x8A    // Display Start Line (2/3 byte)                          // !=
#define LCD_CMD_DISPLAY_MODE        0x66    // Display Mode (2 byte): 0=4 gray-scale, 1=binary        // !=
#define LCD_CMD_GRAY_SCALE_PATTERN  0x39    // Gray Scale Pattern (2 byte)                            // !=
#define LCD_CMD_COM_OUT_STATUS      0xC4    // Common Output Status Select: 0=Normal/1=Reverse        // !=
#define LCD_CMD_DUTY_SET            0x6D    // Duty Set Command (3 byte)
#define LCD_CMD_BUILTIN_OSC         0xAA    // Built-in Oscillator Circuit: [0]:0=Off/1=On
#define LCD_CMD_BUILTIN_OSC_FREQ    0x5F    // Built-in Oscillator Circuit Frequency Select (2 byte)
#define LCD_CMD_PWR_CONTROL         0x25    // Power Control Set (2 byte)
#define LCD_CMD_LC_DRIVE_VOLTAGE    0x2B    // Liquid Crystal Drive Voltage Select (2 byte)
#define LCD_CMD_EL_VOLUME           0x81    // Electronic Volume Set (2 byte)
#define LCD_CMD_RESET               0xE2    // Software Reset


ST7565::ST7565(proto_t displayproto, int Hz, PinName mosi, PinName miso, PinName sclk, PinName CS, PinName reset, PinName DC, const char *name, unsigned int LCDSIZE_X, unsigned  int LCDSIZE_Y)
    : LCD(displayproto, Hz, mosi, miso, sclk, CS, reset, DC, LCDSIZE_X, LCDSIZE_Y, IC_X_SEGS, IC_Y_COMS, name)
{
    hw_reset();
    BusEnable(true);
    init();
    cls();
    set_orientation(1);
    locate(0,0);
}
// reset and init the lcd controller
// init sequence is manufacturer specific
void ST7565::init()
{
    wr_cmd8(LCD_CMD_RESET);   //  sw reset
    wait_ms(10);
    
    wr_cmd8(LCD_CMD_BUILTIN_OSC_FREQ);   // Init Sequence
    wr_data8(0x00);
    wr_cmd8(LCD_CMD_BUILTIN_OSC|1);
    wr_cmd8(LCD_CMD_LC_DRIVE_VOLTAGE);
    wr_data8(0x07);
    wr_cmd8(LCD_CMD_EL_VOLUME);
    wr_data8(0x0D);
    wr_cmd8(LCD_CMD_PWR_CONTROL);
    wr_data8(0x1F);
    wr_cmd8(LCD_CMD_DUTY_SET);
    wr_data8(0x11);
    wr_data8(0x00);
    wr_cmd8(LCD_CMD_DISPLAY_REVERSE|1);
    wr_cmd8(LCD_CMD_DISPLAY_ALL_LIGHT|0);
    wr_cmd8(LCD_CMD_DISPLAY_MODE);
    wr_data8(0x00);     // 4 grayscale mode
//    wr_data8(0x01);     // binary mode
    wr_cmd8(LCD_CMD_GRAY_SCALE_PATTERN);
    wr_data8(0x43);
    wr_cmd8(LCD_CMD_COM_OUT_STATUS|1);
    wr_cmd8(LCD_CMD_DISPLAY_START_LINE);
    wr_data8(0);
    wr_cmd8(LCD_CMD_DISPLAY_OFFON|1);
}